"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const openApiGenerator_1 = __importDefault(require("./openApiGenerator"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const fs_1 = require("fs");
const util_1 = require("util");
class ServerlessOpenapiGenerator {
    constructor(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.commands = {
            openapi: {
                usage: 'Generate OpenAPI Documentation',
                lifecycleEvents: ['serverless'],
                options: {
                    output: {
                        usage: 'Output file location [default: openapi.yml|json]',
                        shortcut: 'o',
                        type: 'string',
                    },
                    format: {
                        usage: 'OpenAPI file format (yml|json) [default: yml]',
                        shortcut: 'f',
                        type: 'string',
                    },
                },
            },
        };
        this.hooks = {
            'openapi:serverless': this.generate.bind(this),
        };
    }
    log(message, options) {
        this.serverless.cli.log(message, '', options);
    }
    handleCliInput() {
        const formats = ['yaml', 'json'];
        const defaultOutputFilename = 'openapi.def';
        const config = {
            format: 'yaml',
            output: `${defaultOutputFilename}.yml`,
        };
        config.format = this.serverless.pluginManager.cliOptions.format || 'yaml';
        if (!formats.includes(config.format.toLowerCase()))
            throw new Error(`Invalid format option specified. Please choose either "yaml" or "json"`);
        config.output =
            this.serverless.pluginManager.cliOptions.output || config.format === 'json'
                ? `${defaultOutputFilename}.json`
                : `${defaultOutputFilename}.yml`;
        return config;
    }
    async generate() {
        this.log('Generating OpenAPI specification file');
        const documentation = this.serverless.service.custom.openapi;
        if (!documentation.openapi)
            throw new Error(`You must have an "openapi" field with the target OpenAPI version.`);
        if (!documentation.info || !documentation.info.title || !documentation.info.version)
            throw new Error(`You must have an "info" field with "title" and "version" properties`);
        const generator = new openApiGenerator_1.default(documentation);
        if (documentation.components)
            generator.parseComponents(documentation.components);
        const functions = this.serverless.service.functions;
        generator.parseFunctions(functions);
        try {
            const validationResults = await generator.validate();
            this.log(util_1.inspect(validationResults, false, null), { bold: true });
            if (validationResults.filter((result) => result.severity === 'Error').length > 0) {
                return;
            }
            const cliOptions = this.handleCliInput();
            const output = cliOptions.format === 'yaml'
                ? js_yaml_1.default.dump(generator.definition)
                : JSON.stringify(generator.definition);
            fs_1.writeFileSync(cliOptions.output, output);
            this.log(`OpenAPI Generation Successful. Written to file ${cliOptions.output}`, {
                color: 'green',
                bold: true,
            });
        }
        catch (err) {
            this.log(err.message, { bold: true, color: 'red' });
        }
    }
}
module.exports = ServerlessOpenapiGenerator;
//# sourceMappingURL=index.js.map