"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const spectral_1 = require("@stoplight/spectral");
const utils_1 = require("./utils");
class OpenApiGenerator {
    constructor(schema) {
        this.spectral = new spectral_1.Spectral();
        this.spectral.registerFormat('oas2', spectral_1.isOpenApiv2);
        this.spectral.registerFormat('oas3', spectral_1.isOpenApiv3);
        this.spectral.registerFormat('json-schema-draft4', spectral_1.isJSONSchemaDraft4);
        const { openapi, info, servers = [], components, security = [], tags, externalDocs } = schema;
        this.definition = {
            openapi,
            info,
            paths: {},
        };
        if (servers)
            this.definition.servers = servers;
        if (components)
            this.definition.components = components;
        if (security)
            this.definition.security = security;
        if (tags)
            this.definition.tags = tags;
        if (externalDocs)
            this.definition.externalDocs = externalDocs;
    }
    parseComponents(components) {
        if (components.schemas) {
            const schemaComponents = {};
            const schemas = components.schemas;
            Object.keys(schemas).map((key) => {
                schemaComponents[key] = this.cleanSchema(schemas[key]);
            });
            this.definition.components = {
                ...this.definition.components,
                schemas: { ...schemaComponents },
            };
        }
    }
    parseFunctions(functions) {
        const paths = {};
        Object.keys(functions).map((key) => {
            const httpEvents = functions[key].events
                .filter((event) => !!event.http)
                .map((event) => event.http);
            httpEvents.map((event) => {
                const { summary, description, tags, externalDocs, operationId, parameters, requestBody, responses, callbacks, deprecated, security, servers, } = event.openapi;
                const pathOperation = {};
                if (summary)
                    pathOperation.summary = summary;
                if (description)
                    pathOperation.description = description;
                if (tags)
                    pathOperation.tags = tags;
                if (externalDocs)
                    pathOperation.externalDocs = externalDocs;
                if (parameters)
                    pathOperation.parameters = parameters;
                if (requestBody)
                    pathOperation.requestBody = requestBody;
                if (callbacks)
                    pathOperation.callbacks = callbacks;
                if (deprecated)
                    pathOperation.deprecated = deprecated;
                if (operationId)
                    pathOperation.operationId = operationId;
                if (security)
                    pathOperation.security = security;
                if (servers)
                    pathOperation.servers = servers;
                if (responses)
                    pathOperation.responses = responses;
                paths[event.path] = {
                    ...paths[event.path],
                    [event.method]: pathOperation,
                };
            });
        });
        this.definition.paths = paths;
    }
    cleanSchema(schema) {
        const cleanedSchema = utils_1.clone(schema);
        if (cleanedSchema.$schema)
            delete cleanedSchema.$schema;
        return cleanedSchema;
    }
    async validate() {
        await this.spectral.loadRuleset('spectral:oas');
        const results = await this.spectral.run(this.definition);
        return results.map((result) => {
            const severityOptions = ['Error', 'Warning', 'Information', 'Hint'];
            return { ...result, severity: severityOptions[result.severity] };
        });
    }
}
exports.default = OpenApiGenerator;
//# sourceMappingURL=openApiGenerator.js.map